pub const ALIAS: &str = "Alias";
pub const ASYNC: &str = "Async";
pub const MANUAL: &str = "Manual";
pub const FUNC: &str = "Func";
pub const ONEWAY: &str = "Oneway";
pub const UPDATE: &str = "Update";
pub const QUERY: &str = "Query";
pub const TUPLE: &str = "Tuple";
pub const SERVICE: &str = "Service";
pub const GUARD_RESULT: &str = "GuardResult";
pub const SERVICE_UPDATE: &str = "service_update";
pub const SERVICE_QUERY: &str = "service_query";
pub const STABLE_B_TREE_MAP: &str = "StableBTreeMap";
pub const HEARTBEAT_DECORATOR: &str = "heartbeat";
pub const INIT_DECORATOR: &str = "init";
pub const INSPECT_MESSAGE_DECORATOR: &str = "inspect_message";
pub const POST_UPGRADE_DECORATOR: &str = "post_upgrade";
pub const PRE_UPGRADE_DECORATOR: &str = "pre_upgrade";
pub const QUERY_METHOD_DECORATOR: &str = "query";
pub const UPDATE_METHOD_DECORATOR: &str = "update";
