pub mod dictionary_unpacking_operator_not_supported;
pub mod first_parameter_must_be_self;
pub mod first_parameter_must_not_be_self;
pub mod iterator_unpacking_operator_not_supported;
pub mod param_type_annotation_required;
pub mod return_type_annotation_required;
pub mod too_many_params;

pub use dictionary_unpacking_operator_not_supported::DictionaryUnpackingOperatorNotSupported;
pub use first_parameter_must_be_self::FirstParamMustBeSelf;
pub use first_parameter_must_not_be_self::FirstParamMustNotBeSelf;
pub use iterator_unpacking_operator_not_supported::IteratorUnpackingOperatorNotSupported;
pub use param_type_annotation_required::ParamTypeAnnotationRequired;
pub use return_type_annotation_required::ReturnTypeAnnotationRequired;
pub use too_many_params::TooManyParams;
